//	Programa ejemplos que muestra en una terminal serial configurada a 57600 bauds,
//	las revoluciones por minuto, el sentido de giro del eje del motor u el avance
//	en ticks (pulsos generador por los encoders), para seleccionar un sentido de 
//	giro se mueve el SW1 de acuerdo a la siguiente configuracin:
//	Programa para motor High Torque. Cambiar lnea para relacin de motor de High Torque 
//  Modificar la lnea 176 en el programa dependiendo de la relacin de engranes:
//  Para una relacin de 1:34, la frmula es: rpm =  (4 * delta) / 24
//  Para una relacin de 1:54, la frmula es: rpm = (10 * delta) / 99
//  Para una relacin de 1:36, la frmula es: rpm = (10 * delta) / 66
//
//		____________________________________________________
//		SW1		Sentido de giro
//		_______	____________________________________________
//
//		0(ON)	Giro contrario a las manecillas de reloj	
//		
//		1		Giro en direccion de las manecillas de reloj
//		____________________________________________________
//		
//		
//	Debido a la carga que representan los leds conectados al puerto B, se utliza un 
//	circuito integrado transmisor / receptor de datoS 74LS245 conectado de la siguiente 
//	forma:
//
//																		J6
//		  						  _______ _______
//	  	  ENCODER  		  +5V  --|DIR	 U	  VCC|--  +5V				GND
//								-|A1			E|--  GND				+5V
//	 -------M1					-|A2		   B1|-						INT
//	|		GND--- GND			-|A3		   B2|-						RW
//	|		C1-------------------|A4		   B3|-						RS	Conector 
//	|		C2-------------------|A5		   B4|----------------------B0	para el 
//	|		3.3V-- +5V			-|A6		   B5|----------------------B1	  LCD
//	|	 ---M2					-|A7		   B6|-						B2
//	|	|						-|A8		   B7|-						B3
//	|	|				   GND --|GND		   B8|-						B4
//	|	|  						  _______________				 -------B5
//	|	|														|	 ---B6
//	|	|														|	|	B7
//	|	|			   _____________________________			|	|	|
//	|	|			  |								|			|	|	|
//	|	 -------------|DRIVER				DIR1	|-----------	|	|
//	|				  |DE 					DIR2	|---------------	|
//	 -----------------|POTENCIA				ENABLE	|-------------------
//					  |_____________________________|
//
//	El encoder debe entregar seales en cuadratura, pare el caso de este motor se tienen
//	748 ticks o pulsos por revolucin del eje, el motor tiene una relacin de reduccin 
//	de 1:34
//
//

#include <p18cxxx.h>
#include <delays.h>
#include <stdio.h>		//library sprintf()
#include <stdlib.h>		//libreras atoi(),atof()
#include <usart.h>
#include <Timers.h>
#include <p18f2550.h>

#define DIR_A	PORTBbits.RB5
#define DIR_B	PORTBbits.RB6
#define ENABLE	PORTBbits.RB7
#define GIRO_HORARIO		DIR_A=1; DIR_B=0
#define	GIRO_ANTIHORARIO	DIR_A=0; DIR_B=1
#define Horario		0
#define Antihorario	1
#define SW1  	PORTAbits.RA4

// Declaraciones prototipo de funciones 
void int0_isr(void);  
void timer_isr(void);
// Rutina de servicio de interrupcion
void setup(void); 
void init_18F2550(void);
void delay_ms(int);

// See c018i.c in your C18 compiler dir 
extern void _startup( void ); 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800

void _reset( void ) 
{ 
  _asm goto _startup _endasm 
} 

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808

void _high_ISR(void)
{
  _asm goto int0_isr _endasm
}

#pragma code low_vector = 0x000818
void low_interrupt(void)
{
  _asm goto timer_isr _endasm
}

#pragma code

volatile static long avance=0;			// Nmero de ticks que ha avanzado el eje del motor
volatile static long avance_anterior=0;	// Util para medir las revoluciones por minuto
volatile static unsigned int rpm=0;
volatile static unsigned char sentido_giro=Horario;	// Define el sentido de giro

void main()  
{
	char entero[12];		// Arreglo para representar el avance del eje del motor
	char revoluciones[12];	// Arreglo para representer las revoluciones por minuto

	INTCONbits.INT0IF = 0;		//Bandera de intrrupcion a 0
	INTCONbits.INT0IE = 1;		//Habilitar interrupcion INT0
	INTCON2bits.INTEDG0 = 1;	//Interrupcin se activa en flanco de subida 
	INTCON2bits.RBPU = 0;		//Deshabilita las reistencias de pullup
	TRISBbits.TRISB0 = 1;		
	TRISBbits.TRISB1 = 1;		//Pines PORTB 0 y 1 como entradas
	TRISBbits.TRISB5 = 0;
	TRISBbits.TRISB6 = 0;		
	TRISBbits.TRISB7 = 0;		//Pines PORTB 5,6 y 7 como salidas
	TRISAbits.TRISA4 = 1;

	RCONbits.IPEN = 1;			//Habilta las prioridades en las interrupciones

	WriteTimer0(42099);
	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT & T0_PS_1_256);	//Inicia los parmetros del TIMER0

	INTCONbits.GIE = 1;			//Habilitando interrupciones globales
	INTCONbits.PEIE = 1;		//Habilita las interrupciones de los perifericos
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_ON &
			  USART_ASYNCH_MODE & USART_EIGHT_BIT &
			  USART_CONT_RX & USART_BRGH_LOW, 12);		//SPBRG = 77 PARA 9600 BPS, SPBRG = 12 PARA 57600 BPS 

	ENABLE=1;		// El motor comienza a funcionar
	GIRO_HORARIO;	// en sentido horario
	sentido_giro=Horario;

	while(1)
	{
		if(SW1) {GIRO_HORARIO;} 	// Checa el estado del switch 1 para definir el sentido de giro
		else {GIRO_ANTIHORARIO;}
		itoa(rpm,revoluciones); 	// Convierte un entero a cadena de caracteres
		putsUSART(revoluciones); 
		putrsUSART((const far rom char *)" rpm, ");
		if(rpm==0) putrsUSART((const far rom char *)"eje detenido, ");
		else if(sentido_giro==Horario) putrsUSART((const far rom char *)"sentido horario, ");
		else putrsUSART((const far rom char *)"sentido antihorario, ");
		ltoa(avance, entero); 
		putsUSART(entero); 
		putrsUSART((const far rom char *)" Ticks \n\r");
		delay_ms(300);
	}
} 

#pragma interrupt int0_isr
// Rutina de servicio de interrupcion
void int0_isr() 
{
  	if (INTCONbits.INT0IF)   	//Si la bandera de interrupcin es de INT0 es 1
	{ 
		if(PORTBbits.RB0==PORTBbits.RB1) avance--;
		else avance++;			// Determina el sentido de giro
		INTCON2bits.INTEDG0 = ~INTCON2bits.INTEDG0;	// Cambie el flanco para la proxima interrupcion
		INTCONbits.INT0IF = 0;
  	}                 
}

#pragma interruptlow timer_isr
void timer_isr(void)
{
	static int delta=0;
	if(INTCONbits.TMR0IF)
	{
		delta= avance - avance_anterior; sentido_giro=Horario;
		if(delta < 0){ delta=~delta; sentido_giro=Antihorario;}
		rpm = (10 * delta) / 99;					// Determina las revoluciones por minuto cada medio segundo
		avance_anterior=avance;
		WriteTimer0(42099);						
		INTCONbits.TMR0IF = 0 ;					// Habilita de nuevo la interrupcin
	}
}

void setup()   { 
  init_18F2550();
  PORTB = 0x01;
  // IPEN esta en 1
  RCONbits.IPEN = 1; 	// Habilita las prioridades de las interrupciones
  INTCONbits.PEIE = 1; 
  // bit PERIPHERAL INTERRUPT ENABLE REGISTER igual a 1
  INTCONbits.GIE = 1; 	// Hablilita las interrupciones globales	
}

//Puerto inicializados
void init_18F2550(void)
{
  ADCON1=0x0F; 	// Deshabilita el convertidor analgico a digital
  TRISB=0;		//
  PORTB=0;		// 

  INTCON2bits.RBPU=0;	
  // Selecciona las resistencias de pull-up en el puerto B (Rb4...Rb7).
}

void delay_ms(int i)
{
  long int j;
  for(j=0;j<i;j++)
  {
    Delay1KTCYx(12); 	
    //48 MHZ, DELAY OF 1 MS APPROX.
  }                   
}